@@ -1,12 +1,16 @@
 import argparse
 import traceback
 
-import dicom2nifti
 import logging
+import json
+import nibabel
+import numpy as np
 import os
 import pydeface.utils as pdu
+import SimpleITK as sitk
 import time
 
+from alive_progress import alive_bar
 from ttictoc import Timer
 from fmri_anonymizer.directory_replicator import replicate_structure
 from fmri_anonymizer.file_searcher import walk_through
@@ -15,10 +19,10 @@ from fmri_anonymizer.deidentifier import de_identifier
 from nipype.interfaces.dcm2nii import Dcm2niix
 
 __author__ = "Hugo Angulo"
-__copyright__ = "Copyright 2020, CaosLab"
+__copyright__ = "Copyright 2021, CaosLab"
 __credits__ = ["Hugo Angulo"]
 __license__ = "MIT"
-__version__ = "0.1.3"
+__version__ = "0.2.1"
 __maintainer__ = "Hugo Angulo"
 __email__ = "hugoanda@andrew.cmu.edu"
 __status__ = "Production"
@@ -135,14 +139,16 @@ def main(argv=None):
         if dcm_files.__len__() > 0:
             dcm_folders = dict()
             dcm_recipe = os.path.join(current_path, "util/deid.dicom")
-            for dcm_file in dcm_files:
-                _path, _file = os.path.split(dcm_file)
-                if directory_map[_path] not in dcm_folders:
-                    list_cleaned, percentage_clean = de_identifier(input_folder=_path, recipe_file=dcm_recipe,
-                                                                   output_folder=directory_map[_path])
-                    dcm_cleaned.append(list_cleaned)
-                    dcm_folders[directory_map[_path]] = True
-                    logger.info(f'The current directory has been de-identified {percentage_clean:.2f}%')
+            with alive_bar(len(dcm_files)) as bar:
+                for dcm_file in range(dcm_files):
+                    _path, _file = os.path.split(dcm_file)
+                    if directory_map[_path] not in dcm_folders:
+                        list_cleaned, percentage_clean = de_identifier(input_folder=_path, recipe_file=dcm_recipe,
+                                                                       output_folder=directory_map[_path])
+                        dcm_cleaned.append(list_cleaned)
+                        dcm_folders[directory_map[_path]] = True
+                        logger.info(f'The current directory has been de-identified {percentage_clean:.2f}%')
+                    bar()
 
     logger.info(f'The De-Identification process is done!')
     if deface_flag:
@@ -167,7 +173,9 @@ def main(argv=None):
                 for nifti in nii_de_identified[k]:
                     if nifti.endswith(tuple(nifti_formats)):
                         nii_defaced[nifti] = False
-        # TODO: Deface the niftis.
+        '''
+        Deface the niftis.
+        '''
         for k, v in nii_de_identified.items():
             for each in v:
                 try:
@@ -186,6 +194,50 @@ def main(argv=None):
             traceback.print_stack()
         logger.info(f'The defacing process has finished!')
 
+        '''
+        Here goes the section related to transforming the nifti defaced into dicoms again.
+        '''
+        logger.info('Starting transformation back to DICOM from NIFTI defaced!')
+        logger.info(f'Finding defaced NIFTI files...')
+        nifti_defaced = walk_through(output_path, 'nii', 'nii.gz')
+        logger.info(f'Elements found: {len(nifti_defaced)}')
+        nifti_defaced = list(filter(lambda each: 'defaced' in each, nifti_defaced))
+
+        logger.info(f'Total defaced NIFTI files discovered: {len(nifti_defaced)}')
+        writer = sitk.ImageFileWriter()
+        for k in nifti_defaced:
+            nifti_image = nibabel.load(k)
+            nifti_data = nifti_image.get_fdata()
+            #image_multidimensional = sitk.GetArrayViewFromImage(nifti_image)
+            number_slices = nifti_data.shape[3]
+            logger.info(f'Path of the file to transform: {k}')
+            logger.info(f'Number of slices detected: {number_slices}')
+            path, filename = os.path.split(k)
+            # Here, remove all the current DICOM files
+            dcm_files_anon = walk_through(path, 'dcm')
+            logger.info(f'Now removing initial dicom files found in the path: {path}')
+            logger.info(f'Number of files to remove: {len(dcm_files)}')
+            logger.info(f'Removing now...')
+            with alive_bar(len(dcm_files_anon)) as bar:
+                for each in dcm_files_anon:
+                    os.remove(each)
+                    bar()
+            # Getting all the headers for the DCM
+            json_meta = walk_through(path, 'json')
+            with open(json_meta[0]) as f:
+                metadata = json.load(f)
+            with alive_bar(number_slices) as bar:
+                for slice in range(number_slices):
+                    dicom_filename = path + '/' + filename.split('.nii.gz')[0] + '_' + f'{slice + 1:04}' + '.dcm'
+                    logger.info(f'File to write: {dicom_filename}')
+                    filtered_image = sitk.GetImageFromArray(np.swapaxes(nifti_data[:,:,:,slice], 0, 2).astype(np.uint16))
+                    for k, v in metadata.items():
+                        filtered_image.SetMetaData(str(k), str(v))
+                    writer.SetFileName(dicom_filename)
+                    writer.Execute(filtered_image)
+                    logger.info(f'File written!')
+                    bar()
+
     logger.info(f'Process finished. Time elapsed: {t.stop():.6f}s')
 
 
